function toggleSection(buttonClass, toggleComponent, buttonText) {
    $(toggleComponent).toggle('slow');
    let button = document.getElementById(buttonClass)
    let showText = "Show " + buttonText
    let hideText = "Hide " + buttonText
    button.innerText = (button.innerText === showText ? hideText : showText);
}

function verifyData(activationTabId) {
    //get the input value

    var activationId = $("#activationKey").val();
    var proxyHost = $("#proxyHostname").val();
    var proxyPort = $("#proxyPort").val();
    var proxyUser = $("#proxyUsername").val();
    var proxyPass = $("#proxyPassword").val();
    var activationTab = document.getElementById(activationTabId)
    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            activation: activationId,
            proxyHost: proxyHost,
            proxyPort: proxyPort,
            proxyUser: proxyUser,
            proxyPass: proxyPass
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (data) {

            if (data.success) {
                // activationTab.removeAttribute("hidden");
                activationTab.hidden = false
                activationTab.innerHTML = data.message
                activationTab.style.color = "green"
                var nextButton = document.getElementById("nextButton");
                nextButton.disabled = false
            } else {
                activationTab.hidden = false
                activationTab.innerHTML = data.message
                activationTab.style.color = "red"
                nextButton = document.getElementById("nextButton");
                nextButton.disabled = true
            }
            // activationTab.appendChild(success);
        },
        //on error
        error: function () {
            //bad request
            activationTab.hidden = false
            activationTab.innerHTML = "Your License Key is NOT valid, please enter a valid one"
            activationTab.style.color = "red"
            var nextButton = document.getElementById("nextButton");
            nextButton.disabled = true
        }
    });
}

let lastProduct = ""

function getProductsProject (product) {
    getProject(product)
}
let getProject = function getProjects(product) {
    if ((product === lastProduct)) {
        return
    }
    lastProduct = product
    let ballClip = document.getElementById("inlineLoader");
    ballClip.style.opacity = "1";


    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            type: "getProjects",
            productName: product
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (data) {

            if (!(data.error === undefined)) {
                return;
            }
            var list = $("#projectsList")
            var options = document.getElementById("projectsList").options;
            options.length = 0;
            list.trigger("chosen:updated");

            var projects = data.projects;
            // $("#projectsList").chosen({ search_contains: true });

            projects.forEach(proj => {
                list.append("<option value='" + proj + "'>" + proj + "</option>");
            });

            list.trigger("chosen:updated");
            setTimeout(function () {
                ballClip.style.opacity = "0";
            }, 1000)

            // activationTab.removeAttribute("hidden");
            // activationTab.appendChild(success);
        },
        //on error
        error: function () {
            //bad request
            setTimeout(function () {
                ballClip.style.opacity = "0";
            }, 1000)
        }

    });
};


function clearOptions(listName) {
    let listDocument = document.getElementById(listName);
    if (listDocument.value === "Select All" || listDocument.value === "SELECT_ALL") {
        const selectAllValue = listDocument.value
        const selectedList = $('#' + listName)
        selectedList.val('')
        selectedList.trigger("chosen:updated");
        selectedList.val(selectAllValue)
        selectedList.trigger("chosen:updated");
    }
}

function buildPoliciesMap() {
    let allPolicies = []
    const allPoliciesElements = $('.policy');
    $.map(allPoliciesElements, function (policy) {
        allPolicies[policy.value] = policy.text
    });
    return allPolicies
}

function saveConfiguration() {
    //get the input value
    var wsProduct = $("#productOption").val();
    var wsProject = $("#projectsList").val()
    if (wsProject === null) {
        alert("Please select one or more projects")
        return;
    }
    var policies = $("#policiesList").val();
    if (policies === null) {
        alert("Please select one or more policies")
        return;
    }

    let allPolicies = buildPoliciesMap()
    let jiraProjectObject = document.getElementById("jiraProjects");
    let jiraProjectName = jiraProjectObject.options[jiraProjectObject.selectedIndex].outerText;
    let jiraProjectSanitized = DOMPurify.sanitize(jiraProjectName);

    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            type: "saveRow",
            wsProduct: wsProduct,
            wsProject: wsProject,
            policy: policies,
            jiraProject: jiraProjectSanitized
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (response) {
            if (response.success) {
                buildTableRow(wsProduct, wsProject, policies, jiraProjectName, response.id, allPolicies);
            }
            // clear the selected values of the project and policy (for the new row)
            $('.reset-select').val([]).trigger('chosen:updated')
        },
        //on error
        error: function () {
            //bad request
        }
    });
}

function fetchConfigData() {

    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            type: "fetchConfigData",
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (data, success) {

            let allPolicies = buildPoliciesMap()
            let configurations = data.configEntityDtos;
            configurations.forEach(entity => {
                buildTableRow(entity.product, entity.projects, entity.policies, entity.jiraProject, entity.id,
                    allPolicies)
            })
            // activationTab.removeAttribute("hidden");
            // activationTab.appendChild(success);
        },
        //on error
        error: function () {
            //bad request
        }
    });
}


function changeSelectAllToAll(optionsList) {
    if (optionsList.includes("Select All")) {
        const selectAllIndex = optionsList.indexOf("Select All");
        optionsList[selectAllIndex] = "All"
    }
}

function buildTableRow(wsProduct, wsProject, policy, jiraProject, configEntityId, allPolicies) {

    var confTable = document.getElementById("confTable");
    var row = confTable.insertRow(confTable.rows.length)

    var numberCell = row.insertCell(0);
    numberCell.innerHTML = "" + confTable.rows.length;

    var wsProductCell = row.insertCell();
    wsProductCell.innerHTML = wsProduct;

    var wsProjectCell = row.insertCell();
    changeSelectAllToAll(wsProject);
    wsProjectCell.innerHTML = wsProject.join(", ");

    var wsPolicyCell = row.insertCell();
    let policyNames = []
    policy.forEach(p => {
        policyNames.push(allPolicies[p])
    })
    changeSelectAllToAll(policyNames);
    wsPolicyCell.innerHTML = policyNames.join(", ");

    var jiraProjectCell = row.insertCell();
    jiraProjectCell.innerHTML = jiraProject;


    var deleteCell = row.insertCell();
    deleteCell.innerHTML = "</td><td><input value='Delete' class='addButton' onclick = deleteRow(this," + configEntityId + ")></input></td></tr>"

}

function deleteRow(row, configEntityDB) {
    var result = confirm("Are you sure you want to delete?");

    if (result) {
        $.ajax({
            //the url to send the rowData to
            url: window.location.href + '?'
                + $.param({id: configEntityDB}),
            //the rowData to send to
            data: {type: "deleteRow"},
            //type. for eg: GET, POST
            type: 'DELETE',
            //datatype expected to get in reply form server
            dataType: 'json',
            contentType: 'application/json;charset=utf-8',
            //on success
            success: function (response) {
                if (response.success) {
                    $(row).closest('tr').remove();
                    var confTable = document.getElementById("confTable");
                    for (var i = 0; i < confTable.rows.length; i++) { //fix table indices
                        confTable.rows[i].cells[0].innerHTML = i + 1;
                    }
                }
                // activationTab.removeAttribute("hidden");
                // activationTab.appendChild(success);
            },
            //on error
            error: function () {
                //bad request
            }
        });
    }
}

function defaultProjectSelected() {
    const saveButton = document.getElementById("saveButton");
    saveButton.disabled = false;
    saveButton.style.backgroundColor = "#0747a6";
}

function saveDefaultJiraProject() {
    let defaultJiraProjectObject = document.getElementById("defaultDashboard");
    let jiraProjectName = defaultJiraProjectObject.options[defaultJiraProjectObject.selectedIndex].outerText;
    let jiraProjectSanitized = DOMPurify.sanitize(jiraProjectName);

    if (jiraProjectSanitized == null || jiraProjectSanitized === "") {
        return;
    }
    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            type: "saveDefaultJiraProject",
            jiraProject: jiraProjectSanitized
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (data) {
            const noteElement = $('.selectDefaultProjectNote')[0];
            if ((jiraProjectSanitized == null || jiraProjectSanitized === "") && noteElement !== undefined) {
                noteElement.style.display = 'none';
            }
            setAlert(data, "Default Jira Project '" + jiraProjectSanitized + "' saved successfully", "Could not save default jira project");

        },
        //on error
        error: function (data) {
            setAlert(data, "Default Jira Project '" + jiraProjectName + "' saved successfully", "Could not save default jira project");

        }
    });
}

function submitForm() {
    var loader = document.getElementById('loader');
    loader.style.display = 'flex';
}

function setAlert(data, goodResult, badResult) {
    const connectionResult = $("#alertComponent")
    if (data.success) {
        connectionResult.html(
            "<div id='alertIndication' class=\"alert alert-success\" role=\"alert\">\n" +
            "" + goodResult +
            " </div>")

        setTimeout(function () {
            $("#alertIndication").hide()
        }, 5000)
    } else {
        connectionResult.html(
            "<div class=\"alert alert-danger\" role=\"alert\">\n" +
            "" + badResult + " Message:" + data.message +
            " </div>")
    }
}

function testConnection() {
    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            type: "testConnectionURL",
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (data) {
            setAlert(data, "Connection is ok!", "Connection failed!");
        },
        //on error
        error: function (data) {
            setAlert(data, "Connection is ok!", "Connection failed!");

        }
    });

}

function setIgnoreAlerts() {
    let checked = document.getElementById("ignoreAlertsCheckBox").checked
    $.ajax({
        //the url to send the data to
        url: window.location.href,
        //the data to send to
        data: {
            type: "setIgnoreAlerts",
            ignoreAlerts: checked
        },
        //type. for eg: GET, POST
        type: "POST",
        //datatype expected to get in reply form server
        dataType: "json",
        //on success
        success: function (data) {
            setAlert(data, "Changed ignoreAlerts to " + checked, "Could not save ignoreAlerts");

        },
        //on error
        error: function (data) {
            setAlert(data, "Changed ignoreAlerts to " + checked, "Could not save ignoreAlerts");
        }
    });
}






